"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSServerErrorCodes = exports.TSServerStartStatus = exports.TSServerResponse = void 0;
const tsrequest_1 = require("./tsrequest");
const commons_1 = require("./commons");
const data_1 = require("./data");
const tserrors_1 = require("./tserrors");
class TSServerResponse {
    constructor(response, responseParser) {
        this.response = response;
        this.responseParser = responseParser;
    }
    returnData(data, type = 'application/octet-stream', status = tsrequest_1.Resp.OK) { this.response.writeHead(status, { 'Content-Type': type }); this.response.end(_compatibleData(data)); }
    returnString(s, type = 'text/plain', status = tsrequest_1.Resp.OK) { this.response.writeHead(status, { 'Content-Type': type }); this.response.end((0, commons_1.$value)(s, '')); }
    returnObject() {
        switch (arguments.length) {
            case 0:
                this.returnEmpty();
                break;
            case 1:
                _responseReturnObject(this, true, arguments[0], 'application/json', tsrequest_1.Resp.OK);
                break;
            case 2:
                const ty = (0, commons_1.$isstring)(arguments[1]);
                _responseReturnObject(this, true, arguments[0], ty ? arguments[1] : 'application/json', ty ? tsrequest_1.Resp.OK : arguments[1]);
                break;
            case 3:
                _responseReturnObject(this, true, arguments[0], arguments[1], arguments[2]);
                break;
            default: throw new tserrors_1.TSHttpError('TSServerResponse.returnObject() : Bad arguments', tsrequest_1.Resp.InternalError);
        }
    }
    returnEmpty(status = tsrequest_1.Resp.OK) { this.response.writeHead(status); this.response.end(); }
    returnError() {
        switch (arguments.length) {
            case 0:
                this.returnEmpty(tsrequest_1.Resp.InternalError);
                break;
            case 1:
                const obj = !(0, commons_1.$ok)(arguments[0]) || (0, commons_1.$isobject)(arguments[0]);
                _responseReturnObject(this, true, obj ? arguments[0] : undefined, 'application/json', obj ? tsrequest_1.Resp.InternalError : arguments[0]);
                break;
            case 2:
                _responseReturnObject(this, true, arguments[0], 'application/json', arguments[1]);
                break;
            default: throw new tserrors_1.TSHttpError('TSServerResponse.returnError() : Bad arguments', tsrequest_1.Resp.InternalError);
        }
    }
}
exports.TSServerResponse = TSServerResponse;
var TSServerStartStatus;
(function (TSServerStartStatus) {
    TSServerStartStatus[TSServerStartStatus["AlreadyRunning"] = 0] = "AlreadyRunning";
    TSServerStartStatus[TSServerStartStatus["HTTP"] = 1] = "HTTP";
    TSServerStartStatus[TSServerStartStatus["HTTPS"] = 2] = "HTTPS";
})(TSServerStartStatus = exports.TSServerStartStatus || (exports.TSServerStartStatus = {}));
var TSServerErrorCodes;
(function (TSServerErrorCodes) {
    TSServerErrorCodes[TSServerErrorCodes["MissingMimeType"] = 10000] = "MissingMimeType";
    TSServerErrorCodes[TSServerErrorCodes["UnknownMethod"] = 10001] = "UnknownMethod";
    TSServerErrorCodes[TSServerErrorCodes["ResourceNotFound"] = 10002] = "ResourceNotFound";
    TSServerErrorCodes[TSServerErrorCodes["MethodNotImplemented"] = 10003] = "MethodNotImplemented";
    TSServerErrorCodes[TSServerErrorCodes["InnaccessibleRoot"] = 10004] = "InnaccessibleRoot";
    TSServerErrorCodes[TSServerErrorCodes["BadParameter"] = 10005] = "BadParameter";
    TSServerErrorCodes[TSServerErrorCodes["BadQueryStructure"] = 10006] = "BadQueryStructure";
    TSServerErrorCodes[TSServerErrorCodes["BadBodyStructure"] = 10007] = "BadBodyStructure";
    TSServerErrorCodes[TSServerErrorCodes["MissingBody"] = 10008] = "MissingBody";
    TSServerErrorCodes[TSServerErrorCodes["BadResponseStructure"] = 10009] = "BadResponseStructure";
    TSServerErrorCodes[TSServerErrorCodes["ForbiddenDataResponse"] = 10010] = "ForbiddenDataResponse";
    TSServerErrorCodes[TSServerErrorCodes["BadJSONBody"] = 10011] = "BadJSONBody";
})(TSServerErrorCodes = exports.TSServerErrorCodes || (exports.TSServerErrorCodes = {}));
function _compatibleData(data) {
    if (!(0, commons_1.$ok)(data)) {
        return Buffer.from('');
    }
    if (data instanceof Uint8Array || data instanceof ArrayBuffer) {
        return data;
    }
    return (0, data_1.$bufferFromDataLike)(data);
}
function _responseReturnObject(r, userResponseParser, v, type, status) {
    if ((0, commons_1.$isdataobject)(v)) {
        throw new tserrors_1.TSHttpError('TSServerResponse() : Impossible to return data as object response', tsrequest_1.Resp.InternalError, undefined, TSServerErrorCodes.ForbiddenDataResponse);
    }
    if (userResponseParser && (0, commons_1.$ok)(r.responseParser)) {
        const opts = { errors: [], context: 'json' };
        if (!r.responseParser.validate(v, opts)) {
            throw new tserrors_1.TSHttpError('Invalid structured response', tsrequest_1.Resp.InternalError, { errors: opts.errors }, TSServerErrorCodes.BadResponseStructure);
        }
        v = r.responseParser.rawEncode(v, opts);
    }
    if ((0, commons_1.$ok)(v)) {
        r.response.writeHead(status, { 'Content-Type': type });
        r.response.end(JSON.stringify(v, undefined, 2));
    }
    else {
        r.returnEmpty(status);
    }
}
//# sourceMappingURL=tsserver_types.js.map